love.graphics.setDefaultFilter( "linear", "linear" )

-- IMPORTS
require "assets"
require "timer"
require "star"
require "Player"
require "control"
require "Enemy"
require "Bullet"
require "explosion"
require "levels"
require "anim"

---------------------------------------------------------------------------------------------------- LOAD
function love.load()

	if not love.filesystem.getInfo("starseekerSave.lua") then
		love.filesystem.write("starseekerSave.lua", "highScore = 0")
		highScore = 0
	else
		chunk = love.filesystem.load("starseekerSave.lua")
		chunk()
	end

	-- FULL SCREEN STUFF
	love.mouse.setVisible(false)
	modes = love.window.getFullscreenModes()
	table.sort(modes, function(a, b) return a.width*a.height < b.width*b.height end)   -- sort from smallest to largest
	screenResolutionX = modes[table.getn(modes)].width
	screenResolutionY = modes[table.getn(modes)].height
	scale = (screenResolutionX / 320)
	crtscale = (screenResolutionX / 960)
	love.window.setMode(screenResolutionX, screenResolutionY, {vsync=true, fullscreen=true, fullscreentype="exclusive"})

	-- OPTIONS
	showHitBoxes = false
	hitBoxAlpha = 0.8
	crtalpha = 0.19

	-- FUN
	hardmode = 20

	-- TEMP/DEBUG VARIABLES
	totalobjects = 0
	debugButtons = false
	math.randomseed( os.time() )
	math.random(); math.random(); math.random()

	-- VARIABLES
	pause_current = 1
	pause_tick1 = 0
	pause_tick2 = 1
	pause_quit = {IMG_quit1, IMG_quit2}
	pause_continue = {IMG_continue1, IMG_continue2}
	timers = {}
	bullets = {}
	enemies = {}
	explosions = {}
	ebullets = {}
	p = Player:new()
	g = love.graphics
	xp = 0
	gamestate = "title"
	animations = {}
	crtframes = {IMG_crt1, IMG_crt2, IMG_crt3, IMG_crt4}
	crtcurrent = 1
	crttick = 0
	crtspeed = 0.06
	lifebarframes = {IMG_life1, IMG_life2, IMG_life3, IMG_life4, IMG_life5, IMG_life6, IMG_life7, IMG_life8, IMG_life9}
	xpframes = {IMG_xp1, IMG_xp2, IMG_xp3, IMG_xp4, IMG_xp5, IMG_xp6, IMG_xp7, IMG_xp8, IMG_xp9, IMG_xp10, IMG_xp11}
	lvbarframes = {IMG_lv1, IMG_lv2, IMG_lv3}
	xpbar = 0
	controltick1 = 0
	prompt = 1
	score = 0
	totalScore = 0

	-- FONT STUFF
	font = love.graphics.newImageFont("images/numfont.png", "0123456789", -21)
	love.graphics.setFont(font, 200)
	--font:setFilter( "linear", "linear", 8 )

	-- STARS
	CreateStars()

	--INIT
	initTitle()

end






---------------------------------------------------------------------------------------------------- UPDATE
function love.update(dt)

	if gamestate == "game" then

		-- UPDATE LEVEL
		level.update(dt)

		moveStars(dt)

		-- UPDATE ENEMIES
		for i=1, table.getn(enemies), 1 do
			enemies[i]:update(dt)
		end

		-- MOVE enemy BULLETS
		for i=1, table.getn(ebullets), 1 do
			ebullets[i]:update(dt)
		end

		-- DELETE DEAD enemy BULLETS
		for i=1, table.getn(ebullets), 1 do
			if ebullets[i] then
				if ebullets[i].state == "dead" then
					table.remove(ebullets, i)
					-- REMOVE FROM TOTAL OBJECTS DEBUG VARIABLE
					totalobjects = totalobjects - 1
				end
			end
		end

		-- UPDATE player
		p:update(dt)


		-- REMOVE DEAD ENEMIES
		for i=1, table.getn(enemies), 1 do
			if enemies[i] then
				if enemies[i].deleteIt == true then
					table.remove(enemies, i)
					-- REMOVE FROM TOTAL OBJECTS DEBUG VARIABLE
					totalobjects = totalobjects - 1
				end
			end
		end

		-- CHECK COLLISIONS (player BULLETS + ENEMIES)
		for b=1, table.getn(p.bullets), 1 do
			for e=1, table.getn(enemies), 1 do
				local ax = p.bullets[b].x+p.bullets[b].hitX
				local ay = p.bullets[b].y+p.bullets[b].hitY
				local aw = p.bullets[b].hitW
				local ah = p.bullets[b].hitH
				local bx = enemies[e].x+enemies[e].hitX
				local by = enemies[e].y+enemies[e].hitY
				local bw = enemies[e].hitW
				local bh = enemies[e].hitH
				if CheckCollision(ax, ay, aw, ah, bx, by, bw, bh) then
					if enemies[e].alive then
						enemies[e]:kill()
						xp = xp + 40 -- ADD xp
						p.bullets[b].alive = false
						SND_explosion1:seek(0)
						SND_explosion1:play()
					end
				end
			end
		end

		-- CHECK COLLISIONS (ENEMIES + player)
		for e=1, table.getn(enemies), 1 do
			local ax = p.x+p.hitX
			local ay = p.y+p.hitY
			local aw = p.hitW
			local ah = p.hitH
			local bx = enemies[e].x+enemies[e].hitX
			local by = enemies[e].y+enemies[e].hitY
			local bw = enemies[e].hitW
			local bh = enemies[e].hitH
			if CheckCollision(ax, ay, aw, ah, bx, by, bw, bh) then
				if p.state == "alive" then
					p:kill()
					enemies[e]:kill()
				end
			end
		end

		-- CHECK COLLISIONS (enemy BULLETS + player)
		for b=1, table.getn(ebullets), 1 do
			local ax = p.x+p.hitX
			local ay = p.y+p.hitY
			local aw = p.hitW
			local ah = p.hitH
			local bx = ebullets[b].x+ebullets[b].hitX
			local by = ebullets[b].y+ebullets[b].hitY
			local bw = ebullets[b].hitW
			local bh = ebullets[b].hitH
			if CheckCollision(ax, ay, aw, ah, bx, by, bw, bh) then
				if p.state == "alive" then
					p:kill()
				end
			end
		end

		-- UPDATE EXPLOSIONS
		for i=1, table.getn(explosions), 1 do
			explosions[i]:update(dt)
		end

		-- UPDATE ANIMATIONS
		for i=1, table.getn(animations), 1 do
			animations[i]:update(dt)
		end

		-- REVOVE DEAD ANIMATIONS
		for i=1, table.getn(animations), 1 do
			if animations[i] then
				if animations[i].running == false then
					table.remove(animations, i)
					-- REMOVE TO TOTAL OBJECTS DEBUG VARIABLE
					totalobjects = totalobjects - 1
				end
			end
		end

		-- REMOVE DEAD EXPLOSIONS
		for i=1, table.getn(explosions), 1 do
			if explosions[i] then
				if explosions[i].alive == "dead" then
					table.remove(explosions, i)
					-- REMOVE TO TOTAL OBJECTS DEBUG VARIABLE
					totalobjects = totalobjects - 51
				end
			end
		end

	end

	-- TITLE SCREEN
	if gamestate == "title" then
		moveStars(dt)
		updatePrompt(dt)
	end

	-- PAUSED
	if gamestate == "paused" then
		pause_tick1 = pause_tick1 + dt
		if pause_tick1 > 0.08 then
				pause_tick1 = 0
				if pause_tick2 == 1 then
					pause_tick2 = 2
				else
					pause_tick2 = 1
				end
		end
	end

	if gamestate == "ending" then
		moveStars(dt)
		level.update(dt)

		-- UPDATE ANIMATIONS
		for i=1, table.getn(animations), 1 do
			animations[i]:update(dt)
		end

		-- REVOVE DEAD ANIMATIONS
		for i=1, table.getn(animations), 1 do
			if animations[i] then
				if animations[i].running == false then
					table.remove(animations, i)
					-- REMOVE TO TOTAL OBJECTS DEBUG VARIABLE
					totalobjects = totalobjects - 1
				end
			end
		end
	end

	-- UPDATE TIMERS
	for i=1, table.getn(timers), 1 do
		if timers[i] then timers[i]:update(dt) end
	end

	-- UPDATE CRT FILTER
	crttick = crttick + dt
	if crttick > crtspeed then
		crttick = 0
		if crtcurrent < 4 then
			crtcurrent = crtcurrent + 1
		else
			crtcurrent = 1
		end
	end

	if controlEnabled then checkControls(dt) end

end






---------------------------------------------------------------------------------------------------- DRAW
function love.draw()

	-- DRAW STARS
	for i=1, table.getn(stars), 1 do
		g.setColor(255, 255, 255, stars[i].alpha)
		g.draw(stars[i][stars[i].starType], stars[i].x*scale, stars[i].y*scale, 0, crtscale, crtscale)
		g.setColor(255, 255, 255, 255)
	end

	if gamestate == "game" then


		for i=1, table.getn(enemies), 1 do enemies[i]:draw() end -- DRAW ENEMIES
		for i=1, table.getn(ebullets), 1 do ebullets[i]:draw() end -- DRAW enemy BULLETS
		for i=1, table.getn(explosions), 1 do explosions[i]:draw() end -- DRAW EXPLOSIONS
		p:draw() -- DRAW player

		-- DRAW HIT BOXES
		if showHitBoxes then
			g.setColor(255, 0, 0, hitBoxAlpha)
			for i=1, table.getn(enemies), 1 do
				g.rectangle(
					"fill",
					(enemies[i].x+enemies[i].hitX)*scale,
					(enemies[i].y+enemies[i].hitY)*scale,
					enemies[i].hitW*scale,
					enemies[i].hitH*scale)
			end
			for i=1, table.getn(p.bullets), 1 do
				g.rectangle(
					"fill",
					(p.bullets[i].x+p.bullets[i].hitX)*scale,
					(p.bullets[i].y+p.bullets[i].hitY)*scale,
					p.bullets[i].hitW*scale,
					p.bullets[i].hitH*scale)
			end
			g.rectangle("fill", (p.x+p.hitX)*scale, (p.y+p.hitY)*scale, p.hitW*scale, p.hitH*scale)
			g.setColor(255, 255, 255, 255) -- SET COLOR BACK
		end

		-- LIFE BAR
		if p.lives > 0 then
			if level.asteroids then g.draw(lifebarframes[p.lives+6], 0*scale, 0*scale, 0, crtscale, crtscale)
			else g.draw(lifebarframes[p.lives+1], 0*scale, 0*scale, 0, crtscale, crtscale) end
		end

		-- XP BAR
		if p.lives > 0 then
			if xp > 1000 then xpbar = 10 else xpbar = math.floor(xp/100) end
			if level.asteroids == false then g.draw(xpframes[xpbar+1], 278*scale, 0*scale, 0, crtscale, crtscale) end
		end

		-- LEVEL BAR
		if p.lives > 0 then
			if level.asteroids == false then g.draw(lvbarframes[p.level], 278*scale, 8*scale, 0, crtscale, crtscale) end
		end

		-- SCORE
		if p.lives > 0 then
			g.print(score, 0*scale, 8*scale, 0, crtscale, crtscale)
		end

		-- DRAW ANIMATIONS
		for i=1, table.getn(animations), 1 do if animations[i] then animations[i]:draw() end end

	end

	if gamestate == "title" then

		if titleAnimFrame == 1 then g.setColor(255,255,255,50); g.draw(IMG_title1, -5*scale, -5*scale, 0, crtscale, crtscale); g.setColor(255,255,255,255); end
		if titleAnimFrame == 2 then g.setColor(255,255,255,100); g.draw(IMG_title2, -5*scale, -5*scale, 0, crtscale, crtscale); g.setColor(255,255,255,255); end
		if titleAnimFrame == 3 then g.setColor(255,255,255,150); g.draw(IMG_title3, -5*scale, -5*scale, 0, crtscale, crtscale); g.setColor(255,255,255,255); end
		if titleAnimFrame == 4 then g.setColor(255,255,255,200); g.draw(IMG_title4, -5*scale, -5*scale, 0, crtscale, crtscale); g.setColor(255,255,255,255); end
		if titleAnimFrame == 5 then
			g.setColor(255,255,255,255); g.draw(IMG_title5, -5*scale, -5*scale, 0, crtscale, crtscale)
			g.draw(IMG_topscore, 108*scale, 110*scale, 0, crtscale, crtscale)
			g.print(highScore, 170*scale, 110*scale, 0, crtscale, crtscale) -- PRINTS THE HIGHSCORE
			g.setColor(255,255,255,255)
		end

		if titleScreenPrompt.active then
			if prompt == 1 then
				g.setColor(1, 1, 1, titleScreenPrompt.alpha)
				g.draw(IMG_prompt1, 136*scale, 130*scale, 0, crtscale, crtscale)
				g.setColor(1, 1, 1, 1)
			else
				g.setColor(1, 1, 1, 0.3)
				g.draw(IMG_prompt1, 136*scale, 130*scale, 0, crtscale, crtscale)
				g.setColor(1, 1, 1, 0.3)
			end
			if prompt == 2 then
				g.setColor(1, 1, 1, titleScreenPrompt.alpha)
				g.draw(IMG_prompt2, 139*scale, 139*scale, 0, crtscale, crtscale)
				g.setColor(1, 1, 1, 1)
			else
				g.setColor(1, 1, 1, 0.3)
				g.draw(IMG_prompt2, 139*scale, 139*scale, 0, crtscale, crtscale)
				g.setColor(1, 1, 1, 0.3)
			end
		end
	end

	if gamestate == "paused" then
		if pause_current == 1 then
			g.draw(pause_continue[pause_tick2], 100*scale, 70*scale, 0, crtscale, crtscale)
		else
			g.draw(pause_continue[1], 100*scale, 70*scale, 0, crtscale, crtscale)
		end
		if pause_current == 2 then
			g.draw(pause_quit[pause_tick2], 100*scale, 80*scale, 0, crtscale, crtscale)
		else
			g.draw(pause_quit[1], 100*scale, 80*scale, 0, crtscale, crtscale)
		end
	end

	if gamestate == "ending" then
		-- DRAW ANIMATIONS
		g.print(score, 177*scale, 94*scale, 0, crtscale, crtscale)
		for i=1, table.getn(animations), 1 do if animations[i] then animations[i]:draw() end end
	end

	-- CRT FILTER
	g.setColor(255, 255, 255, crtalpha)
	g.draw(crtframes[crtcurrent], 0, 0, 0, crtscale, crtscale)
	g.setColor(255, 255, 255, 255)

end







----------------------------------------------------------- FUNCTIONS

function CheckCollision(ax1,ay1,aw,ah, bx1,by1,bw,bh)
	local ax2,ay2,bx2,by2 = ax1 + aw, ay1 + ah, bx1 + bw, by1 + bh
	return ax1 < bx2 and ax2 > bx1 and ay1 < by2 and ay2 > by1
end

function clearEnemies()
	enemies = nil; enemies = {}
end

function clearBullets()
	ebullets = nil; ebullets = {}
	p.bullets = nil; p.bullets = {}
end

function clearExplosions()
	explosions = nil; explosions = {}
end

function clearAnimations()
	animations = nil; animations = {}
end

function clearAll()
	clearEnemies()
	clearBullets()
	clearExplosions()
	clearAnimations()
	p:resetPosition()
end

function flushEnemies()
	for i=1, table.getn(enemies), 1 do
		if enemies[i] then
			enemies[i].flushme = true
		end
	end
end

function moveStars(dt)
	for i=1, table.getn(stars), 1 do
		stars[i].y = stars[i].y + stars[i].speed*dt
		if stars[i].y > 200 then
			stars[i].reset(i)
		end
	end
end

function updatePrompt(dt)
	if titleScreenPrompt.counting == "up" then
		if titleScreenPrompt.alpha < 0.9 then
			titleScreenPrompt.alpha = titleScreenPrompt.alpha + dt
		else
			titleScreenPrompt.counting = "down"
		end
	elseif titleScreenPrompt.counting == "down" then
		if titleScreenPrompt.alpha > 0.2 then
			titleScreenPrompt.alpha = titleScreenPrompt.alpha - dt
		else
			titleScreenPrompt.counting = "up"
		end
	end
end

function initTitle()
	controlsEnabled = false
	titleAnimFrame = 0
	timers = nil; timers = {}
	titleScreenPrompt = {active = false, image1 = IMG_prompt1, image2 = IMG_prompt2, alpha = 1, counting = "down", tick = 0}
	logoTimer = timer:new({
		{0.3, 1, "SND_theme:play()"},
		{1.7, 0, ""},
		{0.09, 5, "titleAnimFrame = titleAnimFrame + 1"},
		{0, 1, "titleScreenPrompt.active = true; titleScreenPrompt.alpha = 1; titleScreenPrompt.tick = 0; controlEnabled = true; prompt = 1"}
	})
	table.insert(timers, logoTimer)
end

function initGame()
	gamestate = "game"
	level.currentlevel = level.startinglevel
	level.updateMusic()
	level.init()
	resetScore()
end

function resetToMainScreen()
	controlEnabled = false
	level.running = false
	resetScore()
	gamestate = "title"
	SND_song1:seek(0)
	clearAll()
	starSpeedMin = 20
	starSpeedMax = 130
	p:reset()
	initTitle()
	level.asteroids = false
	level.asteroidsObjects = false
end

function resetScore()
	if score > highScore then
		highScore = score
	end
	score = 0
end







----------------------------------------------------------- FOCUS

function love.focus(f)
	if not f then
		love.mouse.setVisible(true)
		if gamestate == "game" then gamestate = "paused" end
	else
		love.mouse.setVisible(false)
	end
end






------------------------------------------------------------- KEYRELEASED

	--[[
function love.keyreleased(key)
	if key == " " then
		if control_a_isdown then
			control_a_isdown = false
		end
	end
end
]]--
